// ===================
//   FTP MAIN CODE 
// ===================

/*
Version:0.80
Last modified: 21/04/2025
Autor: Rémi Fragnière
www.rfragniere.ch
*/ 
/*

    Script codé en ExtendScript (based on ECMA 3 AKA Javascript 1999)
    https://ae-scripting.docsforadobe.dev
    https://extendscript.docsforadobe.dev
        > Doc de ADOBE
    https://www.goodboy.ninja/snippets/true-after-effects-color-picker
    https://www.goodboy.ninja/snippets/useful-things-to-include-in-your-extendscript-file
        > Quelques petits biscuits des familles
    https://www.daube.ch/docu/fm-documentation/ExtendScript/ScriptUI_for_Dummies-%5BPeterKahrel%5D.pdf
        > PDF avec plein d'explications
    https://scriptui.joonas.me/
        > Pour faire des paneaux automatiquement

*/
// ====================
// ====================

// ============================
//        CHECK SECURITY
// ============================
// En premier on check si les parametres de sécurité sont ok

function AeHasAccessToFilesAndWeb() {
    try {
        var securitySetting = app.preferences.getPrefAsLong(
            "Main Pref Section",
            "Pref_SCRIPTING_FILE_NETWORK_SECURITY"
        );
        return securitySetting == 1;
    } catch (e) {
        // something went wrong, so assume access is denied. You can choose what to do if that happens.
        return false;
    }
}

var userGivenAccess = AeHasAccessToFilesAndWeb(); // true or false
if (!userGivenAccess) {
    alert("Pour utiliser FTP au mieux, il faut que tu lui donnes accès à tes préférences.\n" +
    "VA DANS \"Edition > Préférences > Module de script et Expressions\"\n" +
    "COCHE \"Autoriser les scripts à écrire...\" (le premier)\n" +
    "DECOCHE \"Avertir l'utilisateur...\" (le troisième)\n");
}

// ============================
//      VARIABLES TEMPLATES
// ============================
// Partie qui liste les variables à utiliser au cours du script
try{
    //GUI
    $.evalFile( File("FTP_FILES/Scripts/GuiFunctions.jsx") ); //button size
    //tools
    $.evalFile( File("FTP_FILES/Scripts/magicEaser.jsx") ); //Quick tools magic easer
    $.evalFile( File("FTP_FILES/Scripts/anchorPointController.jsx") ); //anchor Control
    $.evalFile( File("FTP_FILES/Scripts/nullParentFunctions.jsx") ); //null

    //colors
    $.evalFile( File("FTP_FILES/Scripts/paletteGenerator.jsx") ); 

    //LAYERS
    $.evalFile( File("FTP_FILES/Scripts/layerDuration.jsx") );
    $.evalFile( File("FTP_FILES/Scripts/sequenceFunctions.jsx") ); 
    $.evalFile( File("FTP_FILES/Scripts/sortLayerFTP.jsx") ); 

    //COMPS
    $.evalFile( File("FTP_FILES/Scripts/trueDuplicator.jsx") ); 
    $.evalFile( File("FTP_FILES/Scripts/scaleComps.jsx") ); 
    $.evalFile( File("FTP_FILES/Scripts/textImageToComp.jsx") ); 

    //EXPRESSIONS
    $.evalFile( File("FTP_FILES/Scripts/ampAudio.jsx") ); 
    $.evalFile( File("FTP_FILES/Scripts/wiggleExpression.jsx") ); 
    $.evalFile( File("FTP_FILES/Scripts/gravityExpression.jsx") ); 
    $.evalFile( File("FTP_FILES/Scripts/blinkExpr.jsx") ); 
    $.evalFile( File("FTP_FILES/Scripts/timeExpr.jsx") );
    $.evalFile( File("FTP_FILES/Scripts/loopOutExpr.jsx") ); 

    //misc

    //footer
    $.evalFile( File("FTP_FILES/Scripts/projectCleaner.jsx") ); //project cleaner


} catch (e) {
    alert("Erreur lors du chargement des scripts externes : " + e.toString());
}

// ---------------------
//  ELEMENTS GRAPHIQUES
// ---------------------
// On charge les éléments graphiques

try {
    // ICONES


    var FTP_ICON = File("FTP_FILES/Pictos/FTP-logo.png");
    var help_icon = File("FTP_FILES/Pictos/help_icon.png");
    var help_icon_s = File("FTP_FILES/Pictos/help_icon_small.png");
    var SEQ_01 = File("FTP_FILES/Pictos/SeqIcons-01.png");
    var SEQ_02 = File("FTP_FILES/Pictos/SeqIcons-02.png");
    var SEQ_03 = File("FTP_FILES/Pictos/SeqIcons_random.png");
    var SEQ_REVERSE = File("FTP_FILES/Pictos/SeqIcons_reverse.png");

} catch(e) {
   // alert("Oh no..\nThe 'FTP_files' folder is not in the same location as the 'FTP.jsx'. Please correct this in order to use FTP. Consult the documentation if you need assistance.");
}

// ==============
//       UI
// ==============
// Ici on construit la fenêtre de FTP

// ----------
//   PANEL 
// ----------

var tabPannel;
// Variable du panneau, on doit le mettre ici pour pouvoir le récupérer dans toutes les fonctions

var w = this;
// On se définit l'objet de la fenêtre où faire notre affichage, en l'occurrence la popup qui s'affiche quand on ouvre Brigitte
// "w" pour WINDOW
w.orientation= "column";
// default pour les window et panel = Column
// default pour les groupes = Row
w.alignChildren= ["fill", "top"];
// Alignement des éléments (enfants) au sein de cette fenêtre
// [0] alignement Horizontal et [1] alignement Vertical

w.margins = 0;
w.margins.left = 2;
// Marges internes de la fenêtre (en pixels)
w.spacing = 0;
// Marge de séparation des éléments enfants de fenêtre
w.minimumSize.width = 150 ;
w.maximumSize.width = 900 ;
// Limites de taille du conteneur
// -------------
//  GUI VARIABLES
// -------------
var smallInputWidth = 40;
var midInputWidth = 50;
var smallInputHeight = 22;
var largeInputWidth = 100;

var smallButtonWidth = 45;
var midButtonWidth = 65;
var smallButtonHeight = 24;

var smallSquareButtons = 20;

// ----------
//   HEADER
// ----------

var header = w.add("group");
header.alignChildren = ["fill", "top"];
header.margins.top = 5;
header.margins.bottom = 8;
header.margins.left = 2;
header.margins.right = 2;
header.spacing = 10;
header.alignment="left";

var emojiFtpHeader = header.add("image", undefined, ScriptUI.newImage(FTP_ICON, FTP_ICON, FTP_ICON, FTP_ICON) );
emojiFtpHeader.helpTip = "Made by Rémi Fragnière";
// Emoji de sa petite tête
var presentation = header.add ("statictext", undefined, "Fluffy Tools Pack");
presentation.helpTip = "Made by Rémi Fragnière";
// Texte au rollover
presentation.justify = "center"; // pas certain d'avoir besoin de ça
presentation.alignment = "center"; // Centre le texte

// --------
//   BODY
// --------

// TAB PANEL
tabPannel = w.add("tabbedpanel", undefined, "");
tabPannel.alignment = ["fill", "fill"];
tabPannel.spacing = 0;
tabPannel.alignChildren = ["fill", "top"];
tabPannel.margins = 0;
tabPannel.margins.right = 0;

        // ===============
        // == TAB QUICK TOOLS ==
        // ===============
        var tabTOOLS = tabPannel.add("tab", undefined, "TOOLS" );
        tabTOOLS.margins = 0;
        tabTOOLS.margins.left = 10;
        tabTOOLS.margins.top = 8;
        tabTOOLS.spacing = 5;
        tabTOOLS.alignChildren = ["left", "top"];

                // =============================
                // == Magic Easer ==
                // =============================
                var easerPanel = tabTOOLS.add("panel", undefined,"Magic Easer");
                easerPanel.orientation = "column";
                easerPanel.spacing =5;
                easerPanel.alignChildren = ["left", "top"];
                
                // Section Type d'interpolation

                var groupe_interpolation_ME = easerPanel.add("group");
                groupe_interpolation_ME.orientation = "column";
                groupe_interpolation_ME.alignChildren = ["left", "top"];
                groupe_interpolation_ME.add("statictext", undefined, "Type d'interpolation :");

                var groupe_boutons_ME = groupe_interpolation_ME.add("group");
                groupe_boutons_ME.orientation = "row";

                var bouton_lineaire_ME = groupe_boutons_ME.add("button", undefined, "Lineaire");
                var bouton_bezier_ME = groupe_boutons_ME.add("button", undefined, "Bezier");
                var bouton_hold_ME = groupe_boutons_ME.add("button", undefined, "Hold");

                // Section Influence
                var groupe_influence_ME = easerPanel.add("group");
                groupe_influence_ME.orientation = "row";
                groupe_influence_ME.add("statictext", undefined, "Influence :");

                var champ_influence_ME = groupe_influence_ME.add("edittext", undefined, "65");
                champ_influence_ME.size = [60, 20];

                var bouton_application_influence_ME = groupe_influence_ME.add("button", undefined, "Appliquer Influence");

                // Gestion des événements
                bouton_lineaire_ME.onClick = function() {
                    definir_interpolation_ME(KeyframeInterpolationType.LINEAR);
                };

                bouton_bezier_ME.onClick = function() {
                    definir_interpolation_ME(KeyframeInterpolationType.BEZIER);
                };

                bouton_hold_ME.onClick = function() {
                    definir_interpolation_ME(KeyframeInterpolationType.HOLD);
                };

                bouton_application_influence_ME.onClick = function() {
                    var valeur_influence_ME = parseFloat(champ_influence_ME.text);
                    if (!isNaN(valeur_influence_ME)) {
                        definir_influence_ME(valeur_influence_ME);
                    } else {
                        alert("Valeur d'influence non valide !");
                    }
                };

                // =============================
                // == Point d'ancrage ==
                // =============================
                var ancorAndNullGroup = tabTOOLS.add("group");
                ancorAndNullGroup.orientation = "row";
                ancorAndNullGroup.alignChildren = ["left", "top"];
                ancorAndNullGroup.spacing =5;


                var anchorControlPanel = ancorAndNullGroup.add("panel", undefined,"Point d'ancrage");
                anchorControlPanel.orientation = "column";
                anchorControlPanel.spacing =5;
                anchorControlPanel.minimumSize = [100, 100];
                // Ajouter les boutons pour le contrôle du point d'ancrage ici
                // Créer un groupe pour les contrôles de point d'ancrage dans l'onglet "Tools"
                var anchorControlGroup = anchorControlPanel.add("group", undefined);
                anchorControlGroup.orientation = "column";
                anchorControlGroup.alignChildren = ["fill", "center"];
                anchorControlGroup.spacing = 2;

                        // -------------------
                        //  Ajouter les Boutons
                        // -------------------
                        // Ligne supérieure
                        var topGroup = anchorControlGroup.add("group");
                        topGroup.orientation = "row";
                        topGroup.alignChildren = "fill";
                        topGroup.spacing = 2;

                                var btnTopLeft = topGroup.add("button", undefined, "↖");
                                var btnTopCenter = topGroup.add("button", undefined, "↑");
                                var btnTopRight = topGroup.add("button", undefined, "↗");

                        // Ligne du milieu
                        var middleGroup = anchorControlGroup.add("group");
                        middleGroup.orientation = "row";
                        middleGroup.alignChildren = "fill";
                        middleGroup.spacing = 2;

                                var btnMiddleLeft = middleGroup.add("button", undefined, "←");
                                var btnCenter = middleGroup.add("button", undefined, "x");
                                var btnMiddleRight = middleGroup.add("button", undefined, "→");

                        // Ligne inférieure
                        var bottomGroup = anchorControlGroup.add("group");
                        bottomGroup.orientation = "row";
                        bottomGroup.alignChildren = "fill";
                        bottomGroup.spacing = 2;

                            var btnBottomLeft = bottomGroup.add("button", undefined, "↙");
                            var btnBottomCenter = bottomGroup.add("button", undefined, "↓");
                            var btnBottomRight = bottomGroup.add("button", undefined, "↘");

                            // Appliquer la même taille à tous les boutons
                            var allButtons = [btnTopLeft, btnTopCenter, btnTopRight, btnMiddleLeft, btnCenter, btnMiddleRight, btnBottomLeft, btnBottomCenter, btnBottomRight];
                            setButtonSize(allButtons, smallSquareButtons, smallSquareButtons);  // Change ici les valeurs pour ajuster la taille souhaitée

                            var anchorHelpGroup = anchorControlPanel.add("group");
                            anchorHelpGroup.margins.left = 5;
                                    var btnHelpAnchor = anchorHelpGroup.add("iconbutton", [0,0,16,16], help_icon_s, {style: "toolbutton"});
                                    btnHelpAnchor.helpTip = "Contrôle la position du point d'ancrage du calque sélectionné";
                                    
                            // -------------------
                            //  Assignation des fonctions aux boutons
                            // -------------------

                            btnTopLeft.onClick = function() {
                                setAnchorPoint(app.project.activeItem.selectedLayers, [0, 0]);
                            }

                            btnTopCenter.onClick = function() {
                                setAnchorPoint(app.project.activeItem.selectedLayers, [0.5, 0]);
                            }

                            btnTopRight.onClick = function() {
                                setAnchorPoint(app.project.activeItem.selectedLayers, [1, 0]);
                            }

                            btnMiddleLeft.onClick = function() {
                                setAnchorPoint(app.project.activeItem.selectedLayers, [0, 0.5]);
                            }

                            btnCenter.onClick = function() {
                                setAnchorPoint(app.project.activeItem.selectedLayers, [0.5, 0.5]);
                            }

                            btnMiddleRight.onClick = function() {
                                setAnchorPoint(app.project.activeItem.selectedLayers, [1, 0.5]);
                            }

                            btnBottomLeft.onClick = function() {
                                setAnchorPoint(app.project.activeItem.selectedLayers, [0, 1]);
                            }

                            btnBottomCenter.onClick = function() {
                                setAnchorPoint(app.project.activeItem.selectedLayers, [0.5, 1]);
                            }

                            btnBottomRight.onClick = function() {
                                setAnchorPoint(app.project.activeItem.selectedLayers, [1, 1]);
                            }

                //====anchor point end
                


                // ===============
                // == Parentage Null==
                // ===============
                var nullParentPanel = ancorAndNullGroup.add("panel", undefined, "Parentage à Null");
                nullParentPanel.orientation = "column";
                nullParentPanel.alignChildren = ["left", "top"];
                nullParentPanel.spacing = 5;

                // Options de conservation
                var nullOptionsGroup = nullParentPanel.add("group");
                nullOptionsGroup.orientation = "column";
                var nullPreservePosition = nullOptionsGroup.add("checkbox", undefined, "Position");
                nullPreservePosition.value = true;
                var nullPreserveRotation = nullOptionsGroup.add("checkbox", undefined, "Rotation");
                var nullPreserveScale = nullOptionsGroup.add("checkbox", undefined, "Echelle");

                // Bouton de création
                var nullCreateButton = nullParentPanel.add("button", undefined, "Créer Null parent");
                nullCreateButton.onClick = function () {
                    createNullParent(preservePosition.value, preserveRotation.value, preserveScale.value);
                };
                //parentage à un nul end



// ===============
// == TAB GRILLE ==
// ===============
var tabGRILLE = tabPannel.add("tab", undefined, "GRILLE" );
tabGRILLE.margins = 0;
tabGRILLE.margins.left = 10;
tabGRILLE.margins.top = 8;
tabGRILLE.spacing = 5;
tabGRILLE.alignChildren = ["left", "top"];       

    // ===============
    // == grille ==
    // ===============        
    var GRIDGroup = tabGRILLE.add("panel", undefined,"Créer une grille");
    GRIDGroup.orientation = "column";
    GRIDGroup.alignChildren = ["left", "top"];

        var groupeParametresGrid = GRIDGroup.add("group");
        groupeParametresGrid.orientation = "row";
        groupeParametresGrid.alignChildren = ["left", "center"];

        // Nombre de colonnes
        var groupeColonnesGrid = groupeParametresGrid.add("group");
        groupeColonnesGrid.add("statictext", undefined, "Colonnes:");
        var saisieColonnesGrid = groupeColonnesGrid.add("edittext", undefined, "4");
        saisieColonnesGrid.characters = 5;

        // Nombre de lignes
        var groupeLignesGrid = groupeParametresGrid.add("group");
        groupeLignesGrid.add("statictext", undefined, "Lignes:");
        var saisieLignesGrid = groupeLignesGrid.add("edittext", undefined, "4");
        saisieLignesGrid.characters = 5;

        // Gouttière (optionnelle)
        var groupeGouttiereGrid = groupeParametresGrid.add("group");
        groupeGouttiereGrid.add("statictext", undefined, "Gouttières (px):");
        var saisieGouttiereGrid = groupeGouttiereGrid.add("edittext", undefined, "0");
        saisieGouttiereGrid.characters = 5;

        GRIDGroup.add("statictext", undefined, "Marges (px):");

        // Marges (optionnelles)
        var groupeMargesGrid = GRIDGroup.add("group");
        groupeMargesGrid.alignChildren = ["left", "center"];

        groupeMargesGrid.add("statictext", undefined, "Gauche:");
        var saisieMargeGaucheGrid = groupeMargesGrid.add("edittext", undefined, "0");
        saisieMargeGaucheGrid.characters = 3;
        groupeMargesGrid.add("statictext", undefined, "Droite:");
        var saisieMargeDroiteGrid = groupeMargesGrid.add("edittext", undefined, "0");
        saisieMargeDroiteGrid.characters = 3;
        groupeMargesGrid.add("statictext", undefined, "Haut:");
        var saisieMargeHautGrid = groupeMargesGrid.add("edittext", undefined, "0");
        saisieMargeHautGrid.characters = 3;
        groupeMargesGrid.add("statictext", undefined, "Bas:");
        var saisieMargeBasGrid = groupeMargesGrid.add("edittext", undefined, "0");
        saisieMargeBasGrid.characters = 3;

        // Groupe pour les boutons et options
        var groupeButtonsGrid = GRIDGroup.add("group");
        groupeButtonsGrid.orientation = "column";
        groupeButtonsGrid.alignChildren = ["left", "center"];

        // Sous-groupe pour la case à cocher et le bouton "Créer"
        var groupeOptionsButtons = groupeButtonsGrid.add("group");
        groupeOptionsButtons.alignChildren = ["left", "center"];

        // Case à cocher pour supprimer les repères existants
        var checkboxSupprimerReperesGrid = groupeOptionsButtons.add("checkbox", undefined, "Supprimer les repères existants");

        // Bouton pour créer la grille
        var boutonCreerGrid = groupeOptionsButtons.add("button", undefined, "Créer");

        var GRIDDeleteGroup = tabGRILLE.add("panel", undefined,"Supprimer tous les repères");
        GRIDDeleteGroup.orientation = "column";
        GRIDDeleteGroup.alignChildren = ["left", "top"];

        // Bouton pour supprimer la grille
        var boutonSuppGrid = GRIDDeleteGroup.add("button", undefined, "Supprimer les repères");
        boutonSuppGrid.onClick = function() {
            var comp = app.project.activeItem; // Ajoutez cette ligne
            if (comp instanceof CompItem) {
                // Sélectionne la composition active
                app.executeCommand(2276); // Commande pour "Effacer les repères"
                alert("Tous les repères ont été supprimés de la composition active.");
            } else {
                alert("Veuillez sélectionner une composition active.");
            }
        }
        var GRIDDAncorGroup = tabGRILLE.add("panel", undefined,"Créer des repères sur le point d'ancrage");
        GRIDDAncorGroup.orientation = "column";
        GRIDDAncorGroup.alignChildren = ["left", "top"];

        // Bouton pour ajouter des repères sur le point d'ancrage
        var boutonAncrageGrid = GRIDDAncorGroup.add("button", undefined, "Repères sur l'ancrage");

      

        // Appliquer la même taille à tous les boutons
        var gridInputs = [saisieColonnesGrid, saisieLignesGrid, saisieGouttiereGrid, saisieMargeGaucheGrid, saisieMargeDroiteGrid, saisieMargeHautGrid, saisieMargeBasGrid];
        setButtonSize(gridInputs, smallInputWidth, smallInputHeight);

        var gridButtons = [boutonCreerGrid, boutonSuppGrid, boutonAncrageGrid];
        setButtonSize(gridButtons, 180, smallButtonHeight);

        // Fonction lors du clic sur le bouton "Créer"
        boutonCreerGrid.onClick = function() {
            // Récupère les valeurs saisies
            var nbColonnes = parseInt(saisieColonnesGrid.text);
            var nbLignes = parseInt(saisieLignesGrid.text);
            var tailleGouttiere = parseFloat(saisieGouttiereGrid.text);
            var margeGauche = parseFloat(saisieMargeGaucheGrid.text);
            var margeDroite = parseFloat(saisieMargeDroiteGrid.text);
            var margeHaut = parseFloat(saisieMargeHautGrid.text);
            var margeBas = parseFloat(saisieMargeBasGrid.text);
            var supprimerReperes = checkboxSupprimerReperesGrid.value;

            // Vérification des valeurs
            if (isNaN(nbColonnes) || nbColonnes <= 0) {
                alert("Veuillez entrer un nombre valide de colonnes.");
                return;
            }
            if (isNaN(nbLignes) || nbLignes <= 0) {
                alert("Veuillez entrer un nombre valide de lignes.");
                return;
            }
            if (isNaN(tailleGouttiere) || tailleGouttiere < 0) {
                alert("Veuillez entrer une taille de gouttière valide.");
                return;
            }

            // Vérifie si une composition est active
            var comp = app.project.activeItem;
            if (!(comp instanceof CompItem)) {
                alert("Veuillez sélectionner une composition.");
                return;
            }

            // Commence le groupe d'annulation
            app.beginUndoGroup("Créer une Grille Typographique");

            // Supprime les repères existants si l'option est cochée
            if (supprimerReperes) {
                app.executeCommand(2276); // Commande pour "Effacer les repères"
            }

            // Calcul des dimensions disponibles
            var largeurDispo = comp.width - margeGauche - margeDroite - (nbColonnes - 1) * tailleGouttiere;
            var hauteurDispo = comp.height - margeHaut - margeBas - (nbLignes - 1) * tailleGouttiere;

            if (largeurDispo <= 0 || hauteurDispo <= 0) {
                alert("Les marges et gouttières sont trop grandes pour la taille de la composition.");
                app.endUndoGroup();
                return;
            }

            var largeurColonne = largeurDispo / nbColonnes;
            var hauteurLigne = hauteurDispo / nbLignes;

            // === Ajout des repères verticaux (colonnes) ===

            // Repère de marge gauche
            comp.addGuide(1, margeGauche);

            for (var i = 1; i < nbColonnes; i++) {
                // Bord droit de la colonne i (début de la gouttière)
                var xDebutGouttiere = margeGauche + i * largeurColonne + (i - 1) * tailleGouttiere;
                var xFinGouttiere = xDebutGouttiere + tailleGouttiere;

                // Deux repères pour encadrer la gouttière
                comp.addGuide(1, xDebutGouttiere);
                comp.addGuide(1, xFinGouttiere);
            }

            // Repère de marge droite
            comp.addGuide(1, comp.width - margeDroite);

            // === Ajout des repères horizontaux (lignes) ===

            // Repère de marge haute
            comp.addGuide(0, margeHaut);

            for (var j = 1; j < nbLignes; j++) {
                var yDebutGouttiere = margeHaut + j * hauteurLigne + (j - 1) * tailleGouttiere;
                var yFinGouttiere = yDebutGouttiere + tailleGouttiere;

                // Deux repères pour encadrer la gouttière
                comp.addGuide(0, yDebutGouttiere);
                comp.addGuide(0, yFinGouttiere);
            }

            // Repère de marge basse
            comp.addGuide(0, comp.height - margeBas);

            // Fin du groupe d'annulation
            app.endUndoGroup();
        };


        // Fonction pour le bouton "Repères sur l'ancrage"
        boutonAncrageGrid.onClick = function() {
            var comp = app.project.activeItem;
            if (!(comp instanceof CompItem)) {
                alert("Veuillez sélectionner une composition active.");
                return;
            }

            if (comp.selectedLayers.length === 0) {
                alert("Veuillez sélectionner un calque.");
                return;
            }

            var layer = comp.selectedLayers[0]; // Prend le premier calque sélectionné

            app.beginUndoGroup("Ajouter repères sur l'ancrage");

            // Obtient la position du point d'ancrage en coordonnées de composition
            var anchorPoint = layer.property("Anchor Point").value;
            var layerPosition = layer.property("Position").value;
            var anchorInComp = [layerPosition[0], layerPosition[1]];

            // Si le calque est 3D, il faut considérer la transformation complète
            if (layer.threeDLayer) {
                var anchorPointWorld = layer.toComp(anchorPoint);
                anchorInComp = [anchorPointWorld[0], anchorPointWorld[1]];
            }

            // Ajoute un repère vertical à la position x
            comp.addGuide(1, anchorInComp[0]);
            // Ajoute un repère horizontal à la position y
            comp.addGuide(0, anchorInComp[1]);

            app.endUndoGroup();
        }

// ===============
// == TAB COLOR ==
// ===============
var tabCOLOR = tabPannel.add("tab", undefined, "COLOR");
tabCOLOR.margins = 0;
tabCOLOR.margins.left = 10;
tabCOLOR.margins.top = 8;
tabCOLOR.spacing = 5;
tabCOLOR.alignChildren = ["left", "top"];  

var PaletteOptionsGrp = tabCOLOR.add("panel", undefined, "Générer une palette");
PaletteOptionsGrp.orientation = "column";
PaletteOptionsGrp.alignChildren = "left";
PaletteOptionsGrp.spacing = 8;

// Nom de la composition
var paletteGrpCompName = PaletteOptionsGrp.add("group");
paletteGrpCompName.orientation = "row";
paletteGrpCompName.add("statictext", undefined, "Nom de la comp :");
var paletteCompNameInput = paletteGrpCompName.add("edittext", undefined, "Ma palette");
paletteCompNameInput.characters = 20;

// Nombre de couleurs
var paletteGrpNum = PaletteOptionsGrp.add("group");
paletteGrpNum.orientation = "row";
paletteGrpNum.add("statictext", undefined, "Nombre de couleurs:");
var paletteNumColorsInput = paletteGrpNum.add("edittext", undefined, "5");
paletteNumColorsInput.characters = 5;



// Orientation de la palette
var paletteGrpOrientation = PaletteOptionsGrp.add("group");
paletteGrpOrientation.orientation = "row";
paletteGrpOrientation.add("statictext", undefined, "Orientation :");
var paletteOrientationDropdown = paletteGrpOrientation.add("dropdownlist", undefined, ["Horizontal", "Vertical"]);
paletteOrientationDropdown.selection = 0;

// Type de palette
var paletteGrpPaletteType = PaletteOptionsGrp.add("group");
paletteGrpPaletteType.orientation = "row";
paletteGrpPaletteType.add("statictext", undefined, "Type de palette :");
var paletteTypeDropdown = paletteGrpPaletteType.add("dropdownlist", undefined, ["Monochrome","Aléatoire", "Analogique", "Complémentaire", "Triadique", "Tétradique", "Monochrome"]);
paletteTypeDropdown.selection = 0;

// Option pour utiliser une première couleur de référence
var paletteGrpRefColor = PaletteOptionsGrp.add("group");
paletteGrpRefColor.orientation = "row";
var paletteUseRefCheckbox = paletteGrpRefColor.add("checkbox", undefined, "Couleur de référence");
paletteUseRefCheckbox.value = false;
var paletteChooseRefBtn = paletteGrpRefColor.add("button", undefined, "Choisir");
var paletteRefColorText = paletteGrpRefColor.add("statictext", undefined, "Aucune");
paletteRefColorText.preferredSize.width = 100;

// Variable globale pour stocker la première couleur de référence en HSL
var paletteRefColorHSL;

paletteChooseRefBtn.onClick = function() {
    var picked = $.colorPicker();
    if (picked != -1) {
        var r = ((picked >> 16) & 0xFF) / 255;
        var g = ((picked >> 8) & 0xFF) / 255;
        var b = (picked & 0xFF) / 255;
        var rgb = [r, g, b];
        paletteRefColorHSL = rgbToHsl(rgb);
        paletteRefColorText.text = rgbToHex(rgb);
    }
};

// Option pour utiliser une deuxième couleur de référence
var paletteGrpSecondRefColor = PaletteOptionsGrp.add("group");
paletteGrpSecondRefColor.orientation = "row";
var paletteUseSecondRefCheckbox = paletteGrpSecondRefColor.add("checkbox", undefined, "Deuxième Couleur de référence");
paletteUseSecondRefCheckbox.value = false;
var paletteChooseSecondRefBtn = paletteGrpSecondRefColor.add("button", undefined, "Choisir");
var paletteSecondRefColorText = paletteGrpSecondRefColor.add("statictext", undefined, "Aucune");
paletteSecondRefColorText.preferredSize.width = 100;

// Variable globale pour stocker la deuxième couleur de référence en HSL
var paletteSecondRefColorHSL;

paletteChooseSecondRefBtn.onClick = function() {
    var picked = $.colorPicker();
    if (picked != -1) {
        var r = ((picked >> 16) & 0xFF) / 255;
        var g = ((picked >> 8) & 0xFF) / 255;
        var b = (picked & 0xFF) / 255;
        var rgb = [r, g, b];
        paletteSecondRefColorHSL = rgbToHsl(rgb);
        paletteSecondRefColorText.text = rgbToHex(rgb);
    }
};

// Option d'afficher les codes hex
var paletteHexCheckbox = PaletteOptionsGrp.add("checkbox", undefined, "Afficher codes hex");
paletteHexCheckbox.value = true;

// Bouton de génération
var paletteGenerateBtn = PaletteOptionsGrp.add("button", undefined, "Générer la palette");

paletteGenerateBtn.onClick = function() {
    var compName = paletteCompNameInput.text;
    var numColors = parseInt(paletteNumColorsInput.text, 10);
    var swatchWidth = parseInt(200, 10);
    var swatchHeight = parseInt(200, 10);
    var compDuration = parseFloat(10);
    var compFps = parseFloat(30);
    var orientation = paletteOrientationDropdown.selection.text;
    var paletteType = paletteTypeDropdown.selection.text;
    
    if (isNaN(numColors) || numColors <= 0) {
        alert("Veuillez entrer un nombre de couleurs valide.");
        return;
    }
    if (isNaN(swatchWidth) || swatchWidth <= 0 || isNaN(swatchHeight) || swatchHeight <= 0) {
        alert("Veuillez entrer une taille valide pour les swatches.");
        return;
    }
    if (isNaN(compDuration) || compDuration <= 0) {
        alert("Veuillez entrer une durée valide pour la composition.");
        return;
    }
    if (isNaN(compFps) || compFps <= 0) {
        alert("Veuillez entrer un FPS valide pour la composition.");
        return;
    }
    
    app.beginUndoGroup("Générer une palette de couleurs");
    
    var baseHSL = (paletteUseRefCheckbox.value && paletteRefColorHSL !== undefined) ? paletteRefColorHSL : undefined;
    var secondHSL = (paletteUseSecondRefCheckbox.value && paletteSecondRefColorHSL !== undefined) ? paletteSecondRefColorHSL : undefined;
    
    var palette = generatePalette(numColors, paletteType, baseHSL, secondHSL);
    
    var compWidth, compHeight;
    if (orientation === "Horizontal") {
        compWidth = swatchWidth * numColors;
        compHeight = swatchHeight;
    } else {
        compWidth = swatchWidth;
        compHeight = swatchHeight * numColors;
    }
    
    var comp = app.project.items.addComp(compName, compWidth, compHeight, 1, compDuration, compFps);
    
    for (var i = 0; i < numColors; i++) {
        var color = palette[i];
        var solidName = "Swatch " + (i + 1);
        var solidLayer = comp.layers.addSolid(color, solidName, swatchWidth, swatchHeight, 1, compDuration);
        
        if (orientation === "Horizontal") {
            var posX = swatchWidth / 2 + i * swatchWidth;
            var posY = compHeight / 2;
            solidLayer.transform.position.setValue([posX, posY]);
        } else {
            var posX = compWidth / 2;
            var posY = swatchHeight / 2 + i * swatchHeight;
            solidLayer.transform.position.setValue([posX, posY]);
        }
        
    // Si la case "Afficher codes hex" est cochée,
    // on crée un calque texte sur le même emplacement que le swatch affichant son code hex.
    if (paletteHexCheckbox.value) {
        var hexCode = rgbToHex(color);
        var textLayer = comp.layers.addText(hexCode);
        
        // Personnalisation du texte
        var textProp = textLayer.property("Source Text");
        var textDocument = textProp.value;
        textDocument.fontSize = 20;         // Taille de police
        textDocument.fillColor = [1, 1, 1];   // Couleur blanche
        textProp.setValue(textDocument);
        
        // Positionnement du calque texte avec un décalage
        var swatchPos = solidLayer.transform.position.value;
        var offsetX = swatchWidth * 0.2; // 20% de la largeur du swatch
        var offsetY = swatchHeight * 0.35; // 35% de la hauteur du swatch
        textLayer.transform.position.setValue([swatchPos[0] - offsetX, swatchPos[1] - offsetY]);
        
        // Optionnel : parenté le calque texte au calque swatch
        textLayer.parent = solidLayer;
    }
    }
    
    app.endUndoGroup();
    
    alert("Palette générée dans la composition : " + compName);
};

// ===============
// == TAB LAYERS ==
// ===============
var tabLAYERS = tabPannel.add("tab", undefined, "LAYERS" );
tabLAYERS.margins = 0;
tabLAYERS.margins.left = 10;
tabLAYERS.margins.top = 8;
tabLAYERS.spacing = 5;
tabLAYERS.alignChildren = ["left", "top"];

        // =============================
        // == Durée des calques ==
        // =============================

        
        var layerDurGrp = tabLAYERS.add("panel", undefined, "Durée des calques");
        layerDurGrp.orientation = "row";
        layerDurGrp.alignChildren = ["left", "top"];

        layerDurGrp.add("statictext", undefined, "Durée :");

        var layerDurInput = layerDurGrp.add("edittext", undefined, "1");
        layerDurInput.characters = 5;

        // Ajout du groupe de boutons radio pour le toggle (frames ou secondes)
        var toggleGroup = layerDurGrp.add("group");
        toggleGroup.orientation = "column";
        toggleGroup.alignChildren = ["left", "top"];

        var toggleFrames = toggleGroup.add("radiobutton", undefined, "Frames");
        var toggleSeconds = toggleGroup.add("radiobutton", undefined, "Secondes");

        // On coche "Frames" par défaut
        toggleFrames.value = true;

        // Ajoute un bouton pour déclencher l'action
        var layerDurApplyButton = layerDurGrp.add("button", undefined, "GO!");

        // Lorsque le bouton est cliqué
        layerDurApplyButton.onClick = function () {
            var inputValue = parseFloat(layerDurInput.text, 10);
            if (!isNaN(inputValue)) {
                var isFramesMode = toggleFrames.value; // Vérifie quel toggle est activé

                // Si le mode "Frames" est sélectionné, on utilise la fonction avec frames
                if (isFramesMode) {
                    changeLayerDurationByFrames(inputValue);
                } else {
                    // Si le mode "Secondes" est sélectionné, on utilise la fonction en secondes
                    changeLayerDurationBySeconds(inputValue);
                }
            } else {
                alert("Veuillez entrer un nombre valide.");
            }
        };

        var btnHelpLayerDur = layerDurGrp.add("iconbutton", [0,0,16,16], help_icon_s, {style: "toolbutton"});
        btnHelpLayerDur.helpTip = "Redéfinit la durée des calques selectionnés.";

        var layerDurButSize = [layerDurApplyButton];
        setButtonSize(layerDurButSize, smallButtonWidth, smallButtonHeight);  // Change ici les valeurs pour ajuster la taille souhaitée
        
        var layerDurInptSize = [layerDurInput];
        setButtonSize(layerDurInptSize, smallInputWidth, smallInputHeight);  // Change ici les valeurs pour ajuster la taille souhaitée


        // ===============
        // == sequence ==
        // ===============

        var sequenceGroup = tabLAYERS.add("panel", undefined,"Calques de séquence");
        sequenceGroup.orientation = "column";
        sequenceGroup.alignChildren = ["left", "top"];

        // Champ d'entrée pour l'intervalle de frames
        var intervalGroup = sequenceGroup.add("group");

        intervalGroup.add("statictext", undefined, "Intervalle (frames) :");
        var frameIntervalInput = intervalGroup.add("edittext", undefined, "0");
        frameIntervalInput.characters = 5;

        var buttonGrp =sequenceGroup.add("group",undefined);
        // Bouton pour séquencer les calques dans l'ordre normal
        var seqNormalButton = buttonGrp.add("iconbutton", undefined,  SEQ_02);
        seqNormalButton.alignment = ["fill", "top"];
        seqNormalButton.preferredSize.height = 23;
        seqNormalButton.preferredSize.width = 50;
        seqNormalButton.onClick = function() {
            var frameInterval = parseInt(frameIntervalInput.text, 10);
            if (!isNaN(frameInterval)) {
                sequenceLayers("normal", frameInterval);
            } else {
                alert("Veuillez entrer un nombre valide pour l'intervalle.");
            }
        }

        // Bouton pour séquencer les calques dans l'ordre inverse
        var seqReverseButton = buttonGrp.add("iconbutton", undefined,  SEQ_01);
        seqReverseButton.alignment = ["fill", "top"];
        seqReverseButton.preferredSize.height = 23;
        seqReverseButton.preferredSize.width = 50;

        seqReverseButton.onClick = function() {
            var frameInterval = parseInt(frameIntervalInput.text, 10);
            if (!isNaN(frameInterval)) {
                sequenceLayers("reverse", frameInterval);
            } else {
                alert("Veuillez entrer un nombre valide pour l'intervalle.");
            }
        }

        // Bouton pour séquencer les calques dans un ordre aléatoire
        var seqRandomButton = buttonGrp.add("iconbutton", undefined,  SEQ_03);
        seqRandomButton.alignment = ["fill", "top"];
        seqRandomButton.preferredSize.height = 23;
        seqRandomButton.preferredSize.width = 50;
        seqRandomButton.onClick = function() {
            var frameInterval = parseInt(frameIntervalInput.text, 10);
            if (!isNaN(frameInterval)) {
                sequenceLayers("random", frameInterval);
            } else {
                alert("Veuillez entrer un nombre valide pour l'intervalle.");
            }
        }

        var btnHelpSeq = sequenceGroup.add("iconbutton", [0,0,16,16], help_icon_s, {style: "toolbutton"});
        btnHelpSeq.helpTip = "Assistant de calques de séquence. Définir un interval (optionel) et créer la séquence.";

    // ===============
    // == layer sorter ==
    // ===============
        //inverser l'ordre des calques
        var layerSortGroup = tabLAYERS.add("panel", undefined,"réorganiser les calques");
        layerSortGroup.orientation = "column";
        layerSortGroup.alignChildren = ["left", "top"];

         // Bouton pour inverser l'ordre des calques
         var inverserCalquesButton = layerSortGroup.add("button", undefined, "Inverser l'ordre");
         inverserCalquesButton.helpTip = "Inverse l'ordre des calques selectionnés";

         inverserCalquesButton.onClick = function() {
            invertLayersOrder(); // Appelle la fonction LoopOut
        };
        
        var layerSortGroupIn  = layerSortGroup.add("panel", undefined,"trier");

        // Groupe pour le choix de la propriété
        var ddGroup = layerSortGroupIn.add("group");
        ddGroup.orientation = "row";
        ddGroup.add("statictext", undefined, "Propriété :");

        // Ajoutez ici autant de propriétés que vous le souhaitez
        var propertyDropdown = ddGroup.add("dropdownlist", undefined, [
            "Opacité",
            "Échelle (moyenne)",
            "Échelle X",
            "Échelle Y",
            "Rotation (2D)",
            "Rotation X (3D)",
            "Rotation Y (3D)",
            "Rotation Z (3D)",
            "Position (distance à l'origine)",
            "Position X",
            "Position Y",
            "Position Z (3D)",
            "Point d'ancrage X",
            "Point d'ancrage Y",
            "Point d'ancrage Z (3D)",
            "InPoint",
            "OutPoint",
            "Durée (Out - In)"
        ]);
        propertyDropdown.selection = 0; // sélection par défaut
        // Groupe de boutons (Trier Asc / Desc)
        var btnGroup = layerSortGroupIn.add("group");
        btnGroup.orientation = "row";

        var btnAsc = btnGroup.add("button", undefined, "Trier Asc");
        var btnDesc = btnGroup.add("button", undefined, "Trier Desc");

        // Événements
        btnAsc.onClick = function () {
            sortSelectedLayers(propertyDropdown.selection.text, true);
        };
        btnDesc.onClick = function () {
            sortSelectedLayers(propertyDropdown.selection.text, false);
        };


// ===============
// == TAB COMPS ==
// ===============
var tabCOMPS = tabPannel.add("tab", undefined, "COMPS" );
tabCOMPS.margins = 0;
tabCOMPS.margins.left = 10;
tabCOMPS.margins.top = 8;
tabCOMPS.spacing = 5;
tabCOMPS.alignChildren = ["left", "top"];

                // =============================
                // == Duplicateur ==
                // =============================
                
                var duplicatorGrp = tabCOMPS.add("panel", undefined, "Dupliquer la composition");
                duplicatorGrp.orientation = "row";
                // duplicatorGrp.add("statictext", undefined,"loltest");
                
                var duplicatorGrpApplyButton = duplicatorGrp.add("button", undefined, "Dupliquer");
                duplicatorGrpApplyButton.onClick = function() {
                    executeDuplicate();
                };
                var btnHelpDupli = duplicatorGrp.add("iconbutton", [0,0,16,16], help_icon_s, {style: "toolbutton"});
                btnHelpDupli.helpTip = "Duplique la composition selectionnée dans le panneau projet ainsi que les précompositions incluses.";


                                
                // =============================
                // == Scale Composition ==
                // =============================
                var mainScaleGRP = tabCOMPS.add("panel", undefined, "Mise à l'échelle de la composition");
                mainScaleGRP.orientation = "row";
                mainScaleGRP.alignChildren = ["fill", "top"];
                mainScaleGRP.margins = [10, 10, 10, 10];
                mainScaleGRP.spacing = 10;

                var scaleCompGrp = mainScaleGRP.add("group");
                scaleCompGrp.orientation = "column";
                scaleCompGrp.alignChildren = ["fill", "top"];
                scaleCompGrp.margins = [10, 10, 10, 10];
                scaleCompGrp.spacing = 10;

                // Groupe pour les options de mise à l'échelle
                var scaleOptionsGrp = scaleCompGrp.add("group");
                scaleOptionsGrp.orientation = "column";
                scaleOptionsGrp.alignChildren = ["left", "top"];
                scaleOptionsGrp.spacing = 5;

                // Boutons radio pour sélectionner l'option de mise à l'échelle
                var scaleByFactorRadio = scaleOptionsGrp.add("radiobutton", undefined, "Facteur d'échelle");
                scaleByFactorRadio.value = true; // Sélectionné par défaut
                var scaleByWidthRadio = scaleOptionsGrp.add("radiobutton", undefined, "Nouvelle largeur");
                var scaleByHeightRadio = scaleOptionsGrp.add("radiobutton", undefined, "Nouvelle hauteur");

                // Groupe pour le champ de saisie
                var scaleInputGrp = scaleOptionsGrp.add("group");
                scaleInputGrp.orientation = "row";
                scaleInputGrp.alignChildren = ["left", "center"];
                scaleInputGrp.spacing = 5;

                // Champ de saisie pour la valeur
                var scaleValueInput = scaleInputGrp.add("edittext", undefined, "1.0");
                scaleValueInput.characters = 10;
                scaleValueInput.helpTip = "Entrez le facteur d'échelle, la nouvelle largeur ou la nouvelle hauteur.";

                

                // Bouton pour appliquer la mise à l'échelle
                var applyScaleButton = scaleInputGrp.add("button", undefined, "Mettre à l'échelle");

                // Bouton d'aide pour la mise à l'échelle

                var btnHelpScaleComp = scaleInputGrp.add("iconbutton", [0,0,16,16], help_icon_s, {style: "toolbutton"});
                btnHelpScaleComp.helpTip = "Met à l'échelle la composition sélectionnée et tous ses calques. Script d'origine fourni par Adobe.";
               

                // Gestion des événements des boutons radio
                scaleByFactorRadio.onClick = function() {
                    scaleValueInput.text = "1.0";
                };

                scaleByWidthRadio.onClick = function() {
                    var activeItem = app.project.activeItem;
                    if ((activeItem == null) || !(activeItem instanceof CompItem)) {
                        alert("Veuillez sélectionner ou ouvrir une composition d'abord.", "Mise à l'échelle de la composition");
                    } else {
                        scaleValueInput.text = Math.floor(activeItem.width * 1.0).toString();
                    }
                };

                scaleByHeightRadio.onClick = function() {
                    var activeItem = app.project.activeItem;
                    if ((activeItem == null) || !(activeItem instanceof CompItem)) {
                        alert("Veuillez sélectionner ou ouvrir une composition d'abord.", "Mise à l'échelle de la composition");
                    } else {
                        scaleValueInput.text = Math.floor(activeItem.height * 1.0).toString();
                    }
                };

                // Gestion de l'événement du bouton "Mettre à l'échelle"
                applyScaleButton.onClick = function() {
                    try {
                        var activeItem = app.project.activeItem;
                        if ((activeItem == null) || !(activeItem instanceof CompItem)) {
                            alert("Veuillez sélectionner ou ouvrir une composition d'abord.", "Mise à l'échelle de la composition");
                            return;
                        }

                        var value = parseFloat(scaleValueInput.text);
                        if (isNaN(value)) {
                            alert(value + " n'est pas un nombre. Veuillez entrer un nombre.", "Mise à l'échelle de la composition");
                            return;
                        }

                        var new_scale_factor;
                        if (scaleByFactorRadio.value) {
                            new_scale_factor = value;
                        } else if (scaleByWidthRadio.value) {
                            new_scale_factor = value / activeItem.width;
                        } else if (scaleByHeightRadio.value) {
                            new_scale_factor = value / activeItem.height;
                        }

                        if (ScaleCompositions.testNewScale(activeItem, new_scale_factor)) {
                            ScaleCompositions.scaleComposition(activeItem, new_scale_factor);
                            // Réinitialiser les options après mise à l'échelle
                            if (scaleByFactorRadio.value) {
                                scaleValueInput.text = "1.0";
                                scaleByFactorRadio.value = true;
                                scaleByWidthRadio.value = false;
                                scaleByHeightRadio.value = false;
                            } else if (scaleByWidthRadio.value) {
                                scaleValueInput.text = Math.floor(activeItem.width * 1.0).toString();
                            } else if (scaleByHeightRadio.value) {
                                scaleValueInput.text = Math.floor(activeItem.height * 1.0).toString();
                            }
                        } else {
                            alert("La valeur rendra la hauteur ou la largeur hors de la plage 1 à 30000. Veuillez entrer une autre valeur.", "Mise à l'échelle de la composition");
                        }
                    } catch(e) {
                        alert("Erreur dans applyScaleButton.onClick : " + e.toString(), "Erreur");
                    }
                };
                // ===============
                // == TEXT TO COMP ==
                // ===============

                var textToCompPanel = tabCOMPS.add("panel", undefined, "Text to comp");
                textToCompPanel.orientation = "column";
                textToCompPanel.alignChildren = ["fill", "top"];
                textToCompPanel.margins = [10, 10, 10, 10];
                textToCompPanel.spacing = 10;

                var TTCGrpOne = textToCompPanel.add("group", undefined);
                TTCGrpOne.spacing = 5;

                  // Champ pour le nom de la composition
                var TTCcompNameGroup = TTCGrpOne.add("group", undefined);
                TTCcompNameGroup.add("statictext", undefined, "Composition Cible:");
                var TTCcompNameInput = TTCcompNameGroup.add("edittext", undefined, "");
                TTCcompNameInput.characters = 30;

                // Champ pour le texte de remplacement
                var TTCtextGroup = TTCGrpOne.add("group", undefined);
                TTCtextGroup.add("statictext", undefined, "Texte de Remplacement:");
                var TTCreplacementTextInput = TTCtextGroup.add("edittext", undefined, "");
                TTCreplacementTextInput.characters = 30;

                var TTCGrpTwo = textToCompPanel.add("group", undefined);
                TTCGrpTwo.spacing = 5;

                // Champ pour le préfixe
                var TTCprefixGroup = TTCGrpTwo.add("group", undefined);
                TTCprefixGroup.add("statictext", undefined, "Préfixe:");
                var TTCprefixInput = TTCprefixGroup.add("edittext", undefined, "");
                TTCprefixInput.characters = 30;



                // Bouton pour lancer le script
                var TTCrunButton = TTCGrpTwo.add("button", undefined, "Créer");

               
                var btnHelpTTC = TTCGrpTwo.add("iconbutton", [0,0,16,16], help_icon_s, {style: "toolbutton"});
                btnHelpTTC.helpTip = "Crée des Clone de la composition cible en remplaçant le texte impérativement nommée TITRE. Remplace TITRE par le texte de remplacement et renomme la composition (préfixe optionel).";
    

                // Écouteur d'événement pour le bouton
                TTCrunButton.onClick = function() {
                    textToComp(TTCcompNameInput, TTCprefixInput, TTCreplacementTextInput);
                };
                var TTCButtons = [TTCrunButton];
                setButtonSize(TTCButtons, smallButtonWidth, smallButtonHeight);  // Change ici les valeurs pour ajuster la taille souhaitée
                var TTCmainInpts = [TTCcompNameInput, TTCreplacementTextInput, TTCprefixInput ];
                setButtonSize(TTCmainInpts, largeInputWidth, smallInputHeight);  // Change ici les valeurs pour ajuster la taille souhaitée


                // ===============
                // == IMG TO COMP ==
                // ===============
                var ImgToCompPanel = tabCOMPS.add("panel", undefined, "Image to comp");
                ImgToCompPanel.orientation = "column";
                ImgToCompPanel.alignChildren = ["fill", "top"];
                ImgToCompPanel.margins = [10, 10, 10, 10];
                ImgToCompPanel.spacing = 10;

                var IMGTCGrpOne = ImgToCompPanel.add("group", undefined);
                IMGTCGrpOne.spacing = 5;

                // Champ pour le nom de la composition
                var IMGTCcompGroup = IMGTCGrpOne.add("group", undefined);
                IMGTCcompGroup.add("statictext", undefined, "Nom de la Composition:");
                var IMGTCcompNameInput = IMGTCcompGroup.add("edittext", undefined, "");
                IMGTCcompNameInput.characters = 30;

                // Champ pour le préfixe
                var IMGTCprefixGroup = IMGTCGrpOne.add("group", undefined);
                IMGTCprefixGroup.add("statictext", undefined, "Préfixe:");
                var IMGTCprefixInput = IMGTCprefixGroup.add("edittext", undefined, "");
                IMGTCprefixInput.characters = 30;

                // Bouton pour lancer le script
                var IMGTCrunButton = IMGTCGrpOne.add("button", undefined, "Créer");

                var btnHelpIMGTC = IMGTCGrpOne.add("iconbutton", [0,0,16,16], help_icon_s, {style: "toolbutton"});
                btnHelpIMGTC.helpTip = "Crée des Clone de la composition cible en remplaçant une image impérativement nommée IMAGE. Remplace IMAGE par l'image selectionnée dans le panneau projet  et renomme la composition (préfixe optionel).";
    

                // Écouteur d'événement pour le bouton
                IMGTCrunButton.onClick = function() {
                    imageToComp(IMGTCcompNameInput, IMGTCprefixInput);
                };
                var IMGTCButtons = [IMGTCrunButton];
                setButtonSize(IMGTCButtons, smallButtonWidth, smallButtonHeight);  // Change ici les valeurs pour ajuster la taille souhaitée
                var IMGTCmainInpts = [IMGTCcompNameInput, IMGTCprefixInput ];
                setButtonSize(IMGTCmainInpts, largeInputWidth, smallInputHeight);  // Change ici les valeurs pour ajuster la taille souhaitée

// ===============
// == TAB EXPRESSION ==
// ===============
var tabEXPRESSION = tabPannel.add("tab", undefined, "EXPRESSION" );
tabEXPRESSION.margins = 0;
tabEXPRESSION.margins.left = 10;
tabEXPRESSION.margins.top = 8;
tabEXPRESSION.spacing = 5;
tabEXPRESSION.alignChildren = ["left", "top"];


        // ===============
        // == wiggle ==
        // ===============

        // Champs d'entrée pour les valeurs de l'expression
        var wiggleExprPanel = tabEXPRESSION.add("panel", undefined, "Wiggle expression");
        wiggleExprPanel.orientation = "column";
        wiggleExprPanel.alignChildren = ["fill", "top"];
        wiggleExprPanel.margins = 10;
        wiggleExprPanel.spacing = 5;

        var wiggleInptGrp = wiggleExprPanel.add("group", undefined);

            // Fréquence
            var wiggleFreqGroup = wiggleInptGrp.add("group");
            wiggleFreqGroup.add("statictext", undefined, "Fréquence :");
            var wiggleFreqInput = wiggleFreqGroup.add("edittext", undefined, "1");
            wiggleFreqInput.characters = 5;

            // Amplitude
            var wiggleAmpGroup = wiggleInptGrp.add("group");
            wiggleAmpGroup.add("statictext", undefined, "Amplitude :");
            var wiggleAmpInput = wiggleAmpGroup.add("edittext", undefined, "110");
            wiggleAmpInput.characters = 5;

            // Durée de la boucle
            var wiggleLoopTimeGroup = wiggleInptGrp.add("group");
            wiggleLoopTimeGroup.add("statictext", undefined, "Durée de boucle (Secs) :");
            var wiggleLoopTimeInput = wiggleLoopTimeGroup.add("edittext", undefined, "3");
            wiggleLoopTimeInput.characters = 5;

            var unifiedWiggleTogGroup = wiggleExprPanel.add("group");

            unifiedWiggleTogGroup.add("statictext", undefined, "Unifié :");
            var unifiedWiggleGroup = unifiedWiggleTogGroup.add("checkbox", undefined, "Oui");

            // Bouton pour appliquer l'expression
            var applyExprButtonWiggle = unifiedWiggleTogGroup.add("button", undefined, "Wiggle");

            // Fonction appelée lorsque le bouton est cliqué
            applyExprButtonWiggle.onClick = function() {
                var freq = parseFloat(wiggleFreqInput.text);
                var amp = parseFloat(wiggleAmpInput.text);
                var loopTime = parseFloat(wiggleLoopTimeInput.text);

                if (!isNaN(freq) && !isNaN(amp) && !isNaN(loopTime)) {

                    //here
                    var applyUnifiedWiggle = unifiedWiggleGroup.value;
                    if(applyUnifiedWiggle){
                        applyUnifiedWiggleExpression(freq, amp, loopTime);
                    }else{
                        applyWiggleExpression(freq, amp, loopTime);
                    }
                    

                } else {
                    alert("Veuillez entrer des valeurs valides pour la fréquence, l'amplitude, et la durée de la boucle.");
                }
            };

            var btnHelpWiggle = unifiedWiggleTogGroup.add("iconbutton", [0,0,16,16], help_icon_s, {style: "toolbutton"});
            btnHelpWiggle.helpTip = "Applique un wiggle sur la propriété selectionnée.";

            var wiggleInpts = [wiggleFreqInput, wiggleAmpInput, wiggleLoopTimeInput ];
            setButtonSize(wiggleInpts, smallInputWidth, smallInputHeight);  // Change ici les valeurs pour ajuster la taille souhaitée

            var wiggleBtns = [applyExprButtonWiggle];
            setButtonSize(wiggleInpts, smallButtonWidth, smallButtonHeight);  // Change ici les valeurs pour ajuster la taille souhaitée

            
        // ===============
        // == loopout ==
        // ===============

         // Champs d'entrée pour les valeurs de l'expression
         var loopOutExprPanel = tabEXPRESSION.add("panel", undefined, "LoopOut expression");
         loopOutExprPanel.orientation = "column";
         loopOutExprPanel.alignChildren = ["fill", "top"];
         loopOutExprPanel.margins = 10;
         loopOutExprPanel.spacing = 5;

         var loopOutGroup = loopOutExprPanel.add("group");

        var loopOutButton = loopOutGroup.add("button", undefined, "LoopOut");
        var loopOutPPButton = loopOutGroup.add("button", undefined, "Ping pong");
        var loopOutOffButton = loopOutGroup.add("button", undefined, "offset");
        var loopOutContButton = loopOutGroup.add("button", undefined, "continue");
        loopOutButton.onClick = function() {
            loopOutFunction(); // Appelle la fonction LoopOut
        };
        loopOutPPButton.onClick = function() {
            loopOutPPFunction(); // Appelle la fonction LoopOut
        };

        loopOutOffButton.onClick = function() {
            loopOutOffFunction(); // Appelle la fonction LoopOut
        };
        loopOutContButton.onClick = function() {
            loopOutContFunction(); // Appelle la fonction LoopOut
        };
        var btnHelploopOut = loopOutGroup.add("iconbutton", [0,0,16,16], help_icon_s, {style: "toolbutton"});
        btnHelploopOut.helpTip = "Applique un loopOut sur la composition selectionnée.";

        var loopOutButtons = [loopOutButton, loopOutPPButton, loopOutOffButton, loopOutContButton];
        setButtonSize(loopOutButtons, midButtonWidth, smallButtonHeight);  // Change ici les valeurs pour ajuster la taille souhaitée


        // ===============
        // == time ==
        // ===============

         // Champs d'entrée pour les valeurs de l'expression
         var timeExprPanel = tabEXPRESSION.add("panel", undefined, "time expression");
         timeExprPanel.orientation = "column";
         timeExprPanel.alignChildren = ["fill", "top"];
         timeExprPanel.margins = 10;
         timeExprPanel.spacing = 5;

                // Groupe pour le champ de saisie
                var groupeTimeInput = timeExprPanel.add("group");
                groupeTimeInput.orientation = "row";
                groupeTimeInput.add("statictext", undefined, "Multiplicateur:");
                var saisieTimeMultiplicateur = groupeTimeInput.add("edittext", undefined, "1");
                saisieTimeMultiplicateur.characters = 5;




                // Groupe pour le champ de saisie
                var groupePostTimeInput = timeExprPanel.add("group");
                groupePostTimeInput.orientation = "row";
                // Ajouter une checkbox pour activer la posterisation
                var postWiggle = groupePostTimeInput.add("checkbox", undefined, "Appliquer une posterisation temporelle");
                groupePostTimeInput.add("statictext", undefined, "Valeur:");
                var postTimeInput = groupePostTimeInput.add("edittext", undefined, "12");
                postTimeInput.characters = 5;


                // Bouton pour appliquer l'expression
                var applyTimeExpr =  groupeTimeInput.add("button", undefined, "GO!");

                applyTimeExpr.onClick =  function() {
               
                    var applypostWiggle = postWiggle.value;
                    if(applypostWiggle){
                        timeExprPost(saisieTimeMultiplicateur,postTimeInput);
                    }else{
                        timeExpression(saisieTimeMultiplicateur);
                    }


                }
                var btnHelpTime = groupePostTimeInput.add("iconbutton", [0,0,16,16], help_icon_s, {style: "toolbutton"});
                btnHelpTime.helpTip = "Applique un time * multiplicateur sur la propriété selectionnée.";


        // ===============
        // == blink expr ==
    
        // ===============        
        
        var blinkPanel = tabEXPRESSION.add("panel", undefined, "Blink expression");
        blinkPanel.orientation = "column";
        blinkPanel.alignChildren = ["fill", "top"];
        blinkPanel.margins = 10;
        blinkPanel.spacing = 5;

                var blinkInptsGroup = blinkPanel.add("group");

                var blinkMinGroup = blinkInptsGroup.add("group");
                blinkMinGroup.add("statictext", undefined, "Valeur minimale:");
                var blinkMinInput = blinkMinGroup.add("edittext", undefined, "0");
                blinkMinInput.characters = 5;
            
                var blinkMaxGroup = blinkInptsGroup.add("group");
                blinkMaxGroup.add("statictext", undefined, "Valeur maximale:");
                var blinkMaxInput = blinkMaxGroup.add("edittext", undefined, "100");
                blinkMaxInput.characters = 5;
            
                // Champ d'entrée pour la fréquence
                var blinkFrequencyGroup = blinkInptsGroup.add("group");
                blinkFrequencyGroup.add("statictext", undefined, "Fréquence (Hz):");
                var blinkFrequencyInput = blinkFrequencyGroup.add("edittext", undefined, "2");
                blinkFrequencyInput.characters = 5;

                var blinkBtnsGroup = blinkPanel.add("group");

                // Toggle pour activer ou désactiver la variation aléatoire
                var blinkVariationGroup = blinkBtnsGroup.add("group");
                var addBlinkVariationCheckbox = blinkVariationGroup.add("checkbox", undefined, "Ajouter une variation de clignotement");
            
                // Bouton pour appliquer l'expression
                var applyBlinkButton = blinkBtnsGroup.add("button", undefined, "Appliquer l'expression");
                
                applyBlinkButton.onClick = function() {
                    // Récupère les valeurs des champs d'entrée
                    createBlink(blinkMinInput,blinkMaxInput, blinkFrequencyInput, addBlinkVariationCheckbox);
                };

        
        
        // ===============
        // == gravity ==
        // ===============


        // Champs d'entrée pour les valeurs de l'expression
        var gravityExprPanel = tabEXPRESSION.add("panel", undefined, "Gravity expression");
        gravityExprPanel.orientation = "column";
        gravityExprPanel.alignChildren = ["fill", "top"];
        gravityExprPanel.margins = 10;
        gravityExprPanel.spacing = 5;

            // Groupe pour les paramètres
            var paramGroup = gravityExprPanel.add("group");
            paramGroup.orientation = "column";
            paramGroup.alignChildren = ["fill", "top"];

            // Champ pour la gravité
            var gravityGroup = paramGroup.add("group");
            gravityGroup.orientation = "row";
            gravityGroup.add("statictext", undefined, "Gravité (px/s²) :");
            var gravityInput = gravityGroup.add("edittext", undefined, "500");
            gravityInput.characters = 10;

            // Champs pour la vitesse initiale
            var velocityGroup = gravityGroup.add("group");
            velocityGroup.orientation = "row";
            velocityGroup.add("statictext", undefined, "Vitesse Initiale X (px/s) :");
            var velocityXInput = velocityGroup.add("edittext", undefined, "0");
            velocityXInput.characters = 5;
            velocityGroup.add("statictext", undefined, "Y (px/s) :");
            var velocityYInput = velocityGroup.add("edittext", undefined, "0");
            velocityYInput.characters = 5;

            var gravityBtnsGroup = paramGroup.add("group");


            // Bouton pour appliquer l'expression
            var applyGravButton = gravityBtnsGroup.add("button", undefined, "Appliquer l'expression");

            applyGravButton.onClick = function() {
                gravityExpression(); // Appelle la fonction LoopOut
            };
            var btnHelpGravity = gravityBtnsGroup.add("iconbutton", [0,0,16,16], help_icon_s, {style: "toolbutton"});
            btnHelpGravity.helpTip = "Applique une expression de gravité sur la position du calque selectionné.";

        // ===============
        // == amp audio ==
        // ===============

        var ampAudioGroup = tabEXPRESSION.add("panel", undefined,"Amplitude audio");
        ampAudioGroup.orientation = "row";
        ampAudioGroup.alignChildren = ["left", "center"];

                var ampAudioButton = ampAudioGroup.add("button", undefined, "Créer");
                ampAudioButton.onClick = function() {
                    createAudioAmp(); // Appelle la fonction LoopOut
                };

                var ampAudioSize = [ampAudioButton];

                setButtonSize(ampAudioSize, smallButtonWidth, smallButtonHeight);  // Change ici les valeurs pour ajuster la taille souhaitée

                var btnHelpAmp = ampAudioGroup.add("iconbutton", [0,0,16,16], help_icon_s, {style: "toolbutton"});
                btnHelpAmp.helpTip = "Crée un calque d'amplitude audio.";


// ===============
// == TAB CountDown ==
// ===============
var tabMisc = tabPannel.add("tab", undefined, "Misc" );
tabMisc.margins = 0;
tabMisc.margins.left = 10;
tabMisc.margins.top = 8;
tabMisc.spacing = 5;
tabMisc.alignChildren = ["left", "top"];

        // ===============
        // == COUNTDOWN ==
        // ===============
        var countDownGRP = tabMisc.add("panel", undefined,"Compte à rebourd");
        countDownGRP.orientation = "column";
        countDownGRP.alignChildren = ["left", "top"];
        countDownGRP.spacing = 5;

        var countDownGroupSimple = countDownGRP.add("panel", undefined,"Simple (S)");
        countDownGroupSimple.orientation = "row";
        countDownGroupSimple.alignChildren = ["left", "center"];
        countDownGroupSimple.spacing = 5;

                 // Ajout du champ de saisie pour les secondes
                 countDownGroupSimple.add("statictext", undefined, "Secs :");
                var cdSecondsInputSimple = countDownGroupSimple.add("edittext", undefined, "");
                cdSecondsInputSimple.characters = 10;

                // Bouton Appliquer
                var cdApplyBtnSimple = countDownGroupSimple.add("button", undefined, "Créer", {name: "ok"});


                cdApplyBtnSimple.onClick = function() {
                    var secondsText = cdSecondsInputSimple.text.replace(/^\s+|\s+$/g, '');
            
                    var totalSeconds = parseInt(secondsText, 10);
                    if (isNaN(totalSeconds) || totalSeconds <= 0) {
                        alert("Veuillez entrer un nombre de secondes valide supérieur à zéro.");
                        return;
                    }
            
                    
            
                    createCountdownLayerSimple(totalSeconds);
                };

        var countDownGroupComplex = countDownGRP.add("panel", undefined,"Complex (HH:MM:SS)");
        countDownGroupComplex.orientation = "row";
        countDownGroupComplex.alignChildren = ["left", "center"];
        countDownGroupComplex.spacing = 5;

                countDownGroupComplex.add("statictext", undefined, "Hrs :");
                var cdHoursInput = countDownGroupComplex.add("edittext", undefined, "");
                cdHoursInput.characters = 5;
            
                countDownGroupComplex.add("statictext", undefined, "Mins :");
                var cdMinutesInput = countDownGroupComplex.add("edittext", undefined, "");
                cdMinutesInput.characters = 5;
            
                countDownGroupComplex.add("statictext", undefined, "Secs :");
                var cdSecondsInputc = countDownGroupComplex.add("edittext", undefined, "");
                cdSecondsInputc.characters = 5;

                var cdApplyBtnComplex = countDownGroupComplex.add("button", undefined, "Créer", {name: "ok"});

                cdApplyBtnComplex.onClick = function() {
                    // Lecture des entrées utilisateur
                    var hoursText = cdHoursInput.text.replace(/^\s+|\s+$/g, '');
                    var minutesText = cdMinutesInput.text.replace(/^\s+|\s+$/g, '');
                    var secondsText = cdSecondsInputc.text.replace(/^\s+|\s+$/g, '');
            
                    var hours = hoursText !== '' ? parseInt(hoursText, 10) : null;
                    var minutes = minutesText !== '' ? parseInt(minutesText, 10) : null;
                    var seconds = secondsText !== '' ? parseInt(secondsText, 10) : null;
            
                    if (isNaN(hours)) hours = null;
                    if (isNaN(minutes)) minutes = null;
                    if (isNaN(seconds)) seconds = null;
            
                    var totalSeconds = 0;
                    if (hours !== null) totalSeconds += hours * 3600;
                    if (minutes !== null) totalSeconds += minutes * 60;
                    if (seconds !== null) totalSeconds += seconds;
            
                    if (totalSeconds <= 0) {
                        alert("Veuillez entrer une durée valide.");
                        return;
                    }
            
                    
            
                    // Appel de la fonction pour créer le calque de compte à rebours
                    createCountdownLayer(hours, minutes, seconds, totalSeconds);
                };
                var cdInputs = [cdSecondsInputSimple,cdHoursInput,cdMinutesInput, cdSecondsInputc];
                setButtonSize(cdInputs, smallInputWidth, smallInputHeight);  // Change ici les valeurs pour ajuster la taille souhaitée

                var cdButtons = [cdApplyBtnSimple,cdApplyBtnComplex];
                setButtonSize(cdButtons, smallButtonWidth, smallButtonHeight);  // Change ici les valeurs pour ajuster la taille souhaitée



// ------------
//   FOOTER
// ------------

var footer = w.add("group");
footer.alignment = ["left","bottom"];
footer.orientation = "row";

footer.margins.top = 5;
footer.margins.bottom = 3;
footer.margins.left = 3;
footer.margins.right = 3;

                // Ajout d'un panel pour les scripts
                var scriptRunPanel = footer.add("panel", undefined, "Lancer un script");
                scriptRunPanel.orientation = "row";
                scriptRunPanel.alignChildren = "left";
                scriptRunPanel.alignment = ["fill","bottom"];


                // Bouton pour exécuter un script
                var runScriptButton = scriptRunPanel.add("button", undefined, "Exécuter Script");
                runScriptButton.helpTip = "Cliquez pour exécuter un script spécifique";

                // Fonction appelée lorsqu'on clique sur le bouton
                runScriptButton.onClick = function() {
                    try {
                        // Remplacez le chemin par le script que vous voulez exécuter
                        var scriptFile = File.openDialog("Sélectionner un script à exécuter", "*.jsx");
                        if (scriptFile) {
                            $.evalFile(scriptFile);
                        } else {
                            alert("Aucun fichier sélectionné.");
                        }
                    } catch (e) {
                        alert("Erreur lors de l'exécution du script: " + e.toString());
                    }
                };

                //ajout d'un panel reduire le projet
                var projectCleanerPanel = footer.add("panel", undefined, "Réduire le projet");
                projectCleanerPanel.orientation = "row";
                projectCleanerPanel.alignChildren = "left";
                projectCleanerPanel.alignment = ["fill","bottom"];
                var btnCleanProject = projectCleanerPanel.add("button", undefined, "Réduire le projet");
                
                btnCleanProject.onClick = function(){
                    app.beginUndoGroup("Réduire le projet");
                    
                    if(app.project.selection.length > 0){
                        cleanProject();
                    }else{
                        alert("Veuillez sélectionner au moins une composition ou un élément avant de nettoyer le projet.");
                    }
                
                    app.endUndoGroup();
                };



// ===========
//   LET'S GO
// ===========
// On lance le script ici

w.layout.layout(true);
// Pour pouvoir construire les éléments de la fenêtre intégrée
// il faut rendre le layout de cette fenêtre éditable
// sans cette ligne, on ne pourra pas voir les éléments de la fenêtre (boutons, listes, etc.)

w.onResizing = w.onResize = function(){ w.layout.resize(); }
// En cas de resize de la fenêtre, cette ligne appelle la fonction pour mettre à jour l'agencement

// w.layout.resize();
// lance le resize du délire