function createNullParent(preservePosition, preserveRotation, preserveScale) {
    app.beginUndoGroup("Parentage automatique");
    try {
        var comp = app.project.activeItem;
        if (!(comp instanceof CompItem)) {
            throw new Error("Aucune composition active");
        }

        var layers = comp.selectedLayers;
        if (layers.length === 0) {
            throw new Error("Aucun calque sélectionné");
        }

        // Création du Null
        var nullLayer = comp.layers.addNull();
        nullLayer.name = "Parent Auto";
        nullLayer.moveToBeginning();

        // Déterminer si besoin d'un Null 3D
        var needs3D = false;
        for (var i = 0; i < layers.length; i++) {
            if (layers[i].threeDLayer) {
                needs3D = true;
                break;
            }
        }
        nullLayer.threeDLayer = needs3D;

        // Calcul des moyennes
        var avgPos = [0, 0, 0];
        var avgRot = [0, 0, 0];
        var avgScale = [100, 100, 100];
        var count = layers.length;
        var layerTransforms = [];

        if (preservePosition || preserveRotation || preserveScale) {
            for (var i = 0; i < layers.length; i++) {
                var layer = layers[i];

                var pos = layer.property("ADBE Transform Group").property("ADBE Position").value;
                var rot = layer.property("ADBE Transform Group").property("ADBE Rotate Z").value;
                var scale = layer.property("ADBE Transform Group").property("ADBE Scale").value;

                var rotationData = layer.threeDLayer ? [
                    layer.property("ADBE Transform Group").property("ADBE Rotate X").value,
                    layer.property("ADBE Transform Group").property("ADBE Rotate Y").value,
                    layer.property("ADBE Transform Group").property("ADBE Rotate Z").value
                ] : [0, 0, rot];

                layerTransforms.push({
                    layer: layer,
                    pos: pos,
                    rot: rotationData,
                    scale: scale
                });

                if (preservePosition) {
                    avgPos[0] += pos[0];
                    avgPos[1] += pos[1];
                    if (pos.length > 2) avgPos[2] += pos[2];
                }

                if (preserveRotation) {
                    avgRot[0] += rotationData[0];
                    avgRot[1] += rotationData[1];
                    avgRot[2] += rotationData[2];
                }

                if (preserveScale) {
                    avgScale[0] += scale[0];
                    avgScale[1] += scale[1];
                    if (scale.length > 2) avgScale[2] += scale[2];
                }
            }

            if (preservePosition) for (var i = 0; i < avgPos.length; i++) avgPos[i] /= count;
            if (preserveRotation) for (var i = 0; i < avgRot.length; i++) avgRot[i] /= count;
            if (preserveScale) for (var i = 0; i < avgScale.length; i++) avgScale[i] /= count;
        }

        if (preservePosition) nullLayer.property("ADBE Transform Group").property("ADBE Position").setValue(avgPos);
        if (preserveRotation) {
            if (needs3D) {
                nullLayer.property("ADBE Transform Group").property("ADBE Rotate X").setValue(avgRot[0]);
                nullLayer.property("ADBE Transform Group").property("ADBE Rotate Y").setValue(avgRot[1]);
            }
            nullLayer.property("ADBE Transform Group").property("ADBE Rotate Z").setValue(avgRot[2]);
        }
        if (preserveScale) {
            nullLayer.property("ADBE Transform Group").property("ADBE Scale").setValue(avgScale);
        }

        for (var i = 0; i < layerTransforms.length; i++) {
            var data = layerTransforms[i];
            var layer = data.layer;

            layer.parent = nullLayer;

            if (preservePosition) {
                var posProp = layer.property("ADBE Transform Group").property("ADBE Position");
                var relPos = [
                    data.pos[0] - avgPos[0],
                    data.pos[1] - avgPos[1]
                ];
                if (data.pos.length > 2) relPos.push(data.pos[2] - avgPos[2]);
                posProp.setValue(relPos);
            }

            if (preserveRotation) {
                if (layer.threeDLayer) {
                    layer.property("ADBE Transform Group").property("ADBE Rotate X").setValue(data.rot[0] - avgRot[0]);
                    layer.property("ADBE Transform Group").property("ADBE Rotate Y").setValue(data.rot[1] - avgRot[1]);
                }
                layer.property("ADBE Transform Group").property("ADBE Rotate Z").setValue(data.rot[2] - avgRot[2]);
            }

            if (preserveScale) {
                var scaleProp = layer.property("ADBE Transform Group").property("ADBE Scale");
                var newScale = [
                    (data.scale[0] / avgScale[0]) * 100,
                    (data.scale[1] / avgScale[1]) * 100
                ];
                if (data.scale.length > 2) newScale.push((data.scale[2] / avgScale[2]) * 100);
                scaleProp.setValue(newScale);
            }
        }

    } catch (e) {
        alert(e.message);
    }
    app.endUndoGroup();
}
