function applyWiggleExpression(freq, amp, loopTime) {
    var comp = app.project.activeItem;

    if (comp && comp instanceof CompItem && comp.selectedProperties.length > 0) {
        app.beginUndoGroup("Appliquer Expression Wiggle");

        // Parcourt les propriétés sélectionnées
        for (var i = 0; i < comp.selectedProperties.length; i++) {
            var property = comp.selectedProperties[i];

            // Vérifie si la propriété peut recevoir une expression
            if (property.canSetExpression) {

                // Expression générale
                var expression = 
                "freq = " + freq + ";\n" +
                "amp = " + amp + ";\n" +
                "loopTime = " + loopTime + ";\n" +
                "t = time % loopTime;\n" +
                "wiggle1 = wiggle(freq, amp, 1, 0.5, t);\n" +
                "wiggle2 = wiggle(freq, amp, 1, 0.5, t - loopTime);\n" +
                "linear(t, 0, loopTime, wiggle1, wiggle2);";

                // Applique l'expression
                property.expression = expression;

            } else {
                alert("La propriété sélectionnée ne peut pas recevoir d'expression.");
            }
        }

        app.endUndoGroup();
    } else {
        alert("Veuillez sélectionner au moins une propriété dans une composition active.");
    }
}

function applyUnifiedWiggleExpression(freq, amp, loopTime) {
    var comp = app.project.activeItem;

    if (comp && comp instanceof CompItem && comp.selectedProperties.length > 0) {
        app.beginUndoGroup("Appliquer Expression Wiggle Unifié");

        // Parcourt les propriétés sélectionnées
        for (var i = 0; i < comp.selectedProperties.length; i++) {
            var property = comp.selectedProperties[i];

            // Vérifie si la propriété peut recevoir une expression
            if (property.canSetExpression) {

                // Vérifie si la propriété est multidimensionnelle
                if (property.propertyValueType === PropertyValueType.ThreeD_SPATIAL ||
                    property.propertyValueType === PropertyValueType.ThreeD ||
                    property.propertyValueType === PropertyValueType.TwoD_SPATIAL ||
                    property.propertyValueType === PropertyValueType.TwoD) {

                    // Expression pour un wiggle unifié
                    var expression =
                    "freq = " + freq + ";\n" +
                    "amp = " + amp + ";\n" +
                    "loopTime = " + loopTime + ";\n" +
                    "t = time % loopTime;\n" +
                    "seed = Math.floor(time / loopTime);\n" +
                    "seedRandom(seed, true);\n" +
                    "wiggleVal = wiggle(freq, amp, 1, 0.5, t)[0];\n" +
                    "wiggleVector = [wiggleVal, wiggleVal, wiggleVal];\n" +
                    "wiggle1 = wiggleVector;\n" +
                    "wiggle2 = wiggleVector;\n" +
                    "linear(t, 0, loopTime, wiggle1, wiggle2);";

                    // Applique l'expression
                    property.expression = expression;

                } else {
                    alert("La propriété sélectionnée n'est pas multidimensionnelle.");
                }
            } else {
                alert("La propriété sélectionnée ne peut pas recevoir d'expression.");
            }
        }

        app.endUndoGroup();
    } else {
        alert("Veuillez sélectionner au moins une propriété multidimensionnelle dans une composition active.");
    }
}